/*
 *  mod - a moduleplayer for GUS under the Linux OS (with VoxWare sounddriver)
 *
 *  (C) 1994 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <stdio.h>

#include "mod.h"

/* Global variables */

struct mod_info M;    /* The module */

int main(int argc, char *argv[])
{
    init_options(argc, argv);
    
    while(get_next_module()) {
	init_sound();
	if(load_module(M.filename))
	    play_module();
	
	free_module();
	cleanup_sound();
    }

    return 0;
}
